#include "mpib.h"
#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <string.h>

void MPIB_getopt_help_default(int* exit)
{
	*exit = 0;
}

const char* MPIB_getopt_help_options()
{
	return "h";
}

void MPIB_getopt_help_optarg(int c, const char* program, int* exit)
{
	if (c == 'h')
	{
		fprintf
		(
			stderr,
"usage: mpirun [mpirun options] %s [options]\n\
-h			help\n",
			program
		);
		*exit = 1;
	}
}

void MPIB_getopt_help_bcast(int* exit, int root, MPI_Comm comm)
{
	MPI_Bcast(exit, 1, MPI_INT, root, comm);
}

void MPIB_getopt_precision_default(MPIB_precision* precision)
{
	*precision = (MPIB_precision){5, 100, 0.95, 0.025};
}

const char* MPIB_getopt_precision_options()
{
	return "hr:R:c:e:";
}

void MPIB_getopt_precision_optarg(int c, MPIB_precision* precision)
{
	switch (c)
	{
		case 'h':
			fprintf
			(
				stderr,
"-r I			minimum number of repetitions (default: %d)\n\
-R I			maximum number of repetitions (default: %d)\n\
-c D			confidence level: 0 < D < 1 (default: %le)\n\
-e D			error: 0 < D < 1 (default: %le)\n",
				precision->min_reps,
				precision->max_reps,
				precision->cl,
				precision->eps
			);
			break;
		case 'r':
			precision->min_reps = atoi(optarg);
			if (precision->max_reps < precision->min_reps)
				precision->max_reps = precision->min_reps;
			break;
		case 'R':
			precision->max_reps = atoi(optarg);
			if (precision->min_reps > precision->max_reps)
				precision->min_reps = precision->max_reps;
			break;
		case 'c':
			precision->cl = atof(optarg);
			break;
		case 'e':
			precision->eps = atof(optarg);
			break;
	}
}

void MPIB_getopt_precision_bcast(MPIB_precision* precision, int root, MPI_Comm comm)
{
	MPI_Bcast(precision, sizeof(MPIB_precision), MPI_CHAR, root, comm);
}

void MPIB_getopt_msg_default(int* stride, int* max_size)
{
	*stride = 1024;
	*max_size = 204800;
}

const char* MPIB_getopt_msg_options()
{
	return "hs:m:";
}

void MPIB_getopt_msg_optarg(int c, int* stride, int* max_size)
{
	switch (c)
	{
		case 'h':
			fprintf
			(
				stderr,
"-s I			message size stride (default: %d)\n\
-m I			maximum message size (default: %d)\n",
				*stride,
				*max_size
			);
			break;
		case 's':
			*stride = atoi(optarg);
			break;
		case 'm':
			*max_size = atoi(optarg);
			break;
	}
}

void MPIB_getopt_msg_bcast(int* stride, int* max_size, int root, MPI_Comm comm)
{
	MPI_Bcast(stride, 1, MPI_INT, root, comm);
	MPI_Bcast(max_size, 1, MPI_INT, root, comm);
}

void MPIB_getopt_coll_default(char* operation, char* timing)
{
	strcpy(operation, "");
	strcpy(timing, "max");
}

const char* MPIB_getopt_coll_options()
{
	return "hO:t:";
}

void MPIB_getopt_coll_optarg(int c, char* operation, char* timing)
{
	switch (c)
	{
		case 'h':
			fprintf
			(
				stderr,
"-O S			collective operation (required):\n\
			MPI_Scatter, MPIB_Scatter_linear, MPIB_Scatter_binomial\n\
			MPI_Gather, MPIB_Gather_linear, MPIB_Gather_binomial\n\
			MPI_Scatterv\n\
			MPI_Gatherv\n\
-t S			timing: max, root, global (default: %s)\n",
				timing
			);
			break;
		case 'O':
			strcpy(operation, optarg);
			break;
		case 't':
			strcpy(timing, optarg);
			break;
	}
}

void MPIB_getopt_coll_bcast(char* operation, MPIB_coll_container** container,
	char* timing, MPIB_measure_coll* measure, int root, MPI_Comm comm)
{
	int rank;
	MPI_Comm_rank(comm, &rank);
	int len;
	if (rank == root)
		len = strlen(operation) + 1;
	MPI_Bcast(&len, 1, MPI_INT, root, comm);
	MPI_Bcast(operation, len, MPI_CHAR, root, comm);
	*container = NULL;
	if (!strcmp(operation, "MPI_Scatter"))
		*container = MPIB_Scatter_container_alloc(MPI_Scatter);
	if (!strcmp(operation, "MPIB_Scatter_linear"))
		*container = MPIB_Scatter_container_alloc(MPIB_Scatter_linear);
	if (!strcmp(operation, "MPIB_Scatter_binomial"))
		*container = MPIB_Scatter_container_alloc(MPIB_Scatter_binomial);
	if (!strcmp(operation, "MPI_Gather"))
		*container = MPIB_Gather_container_alloc(MPI_Gather);
	if (!strcmp(operation, "MPIB_Gather_linear"))
		*container = MPIB_Gather_container_alloc(MPIB_Gather_linear);
	if (!strcmp(operation, "MPIB_Gather_binomial"))
		*container = MPIB_Gather_container_alloc(MPIB_Gather_binomial);
	int stride = 256;
	if (!strcmp(operation, "MPI_Scatterv"))
		*container = MPIB_Scatterv_container_alloc(stride);
	if (!strcmp(operation, "MPI_Gatherv"))
		*container = MPIB_Gatherv_container_alloc(stride);
	if (rank == root)
		len = strlen(timing) + 1;
	MPI_Bcast(&len, 1, MPI_INT, root, comm);
	MPI_Bcast(timing, len, MPI_CHAR, root, comm);
	*measure = NULL;
	if (!strcmp(timing, "max"))
		*measure = MPIB_measure_max;
	if (!strcmp(timing, "root"))
		*measure = MPIB_measure_root;
	if (!strcmp(timing, "global"))
		*measure = MPIB_measure_global;
}

void MPIB_getopt_p2p_default(int* parallel)
{
	*parallel = 1;
}

const char* MPIB_getopt_p2p_options()
{
	return "hp:";
}

void MPIB_getopt_p2p_optarg(int c, int* parallel)
{
	switch (c)
	{
		case 'h':
			fprintf
			(
				stderr,
"-p 0/1			parallel p2p benchmarking (default: %d)\n",
				*parallel
			);
			break;
		case 'p':
			*parallel = atoi(optarg);
			break;
	}
}

void MPIB_getopt_p2p_bcast(int* parallel, int root, MPI_Comm comm)
{
	MPI_Bcast(parallel, 1, MPI_INT, root, comm);
}
