#ifndef CPM_GETOPT_H_
#define CPM_GETOPT_H_

/*!
 * \defgroup getopt Options for executables
 * \details As getopt cannot be reused, this module provides an interface for getopt.
 * 
 * Typical parameters of executables are as follows:
 * - \b -h help
 * - \b -O \e S collective operation (required):\n
			MPI_Scatter, MPIB_Scatter_linear, MPIB_Scatter_binomial\n
			MPI_Gather, MPIB_Gather_linear, MPIB_Gather_binomial\n
			MPI_Scatterv\n
			MPI_Gatherv\n
			CPM_Scatter_linear_opt\n
			CPM_Gather_linear_opt\n
			CPM_Scatter_binomial_opt, CPM_Scatter_binomial_cor\n
			CPM_Gather_binomial_opt, CPM_Gather_binomial_cor\n
 * - \b -t \e S timing: max, root, global (default: max)
 * - \b -s \e I message size stride (default: 1024)
 * - \b -m \e I maximum message size (default: 102400)
 * - \b -p 0/1 parallel p2p benchmarking (default: 1)
 * - \b -r \e I minimum number of repetitions (default: 5)
 * - \b -R \e I maximum number of repetitions (default: 100)
 * - \b -c \e D confidence level: 0 < D < 1 (default: 0.95)
 * - \b -e \e D error: 0 < D < 1 (default: 0.025)
 * - \b -M \e S model name (required) 
 * - \b -i \e S	input model file, stdin - standard input (default: none - model will be built)
 * - \b -o \e S	output model file, stdout - standard output (default: stdout)
 * 
 * where:
 * - \e S - string
 * - \e I - integer
 * - \e D - double
 * 
 * \{
 */

void CPM_getopt_coll_optarg(int c, char* operation, char* timing);
void CPM_getopt_coll_bcast(char* operation, MPIB_coll_container** container,
	char* timing, MPIB_measure_coll* measure, int root, MPI_Comm comm);

/*!
 * \}
 */

#endif /*CPM_GETOPT_H_*/
